<?php
defined('ABSPATH') || exit(__('No Access!', 'wip'));

trait WIP_Product_Data {

    protected function get_id($product_id) {
        if (!intval($product_id)) {
            return null;
        }
        $product = wc_get_product($product_id);

        $sku = $product->get_sku();
        if ($this->options->get_option('id-type') == 'sku' && $sku) {
            return $sku;
        }

        return $product_id;
    }

    protected function get_image($product_id) {
        if (!intval($product_id)) {
            return null;
        }
        $product = wc_get_product($product_id);

        if ($product->get_type() == 'variation' && has_post_thumbnail($product_id)) {
            $image = get_the_post_thumbnail($product_id, 'shop_thumbnail');
        } elseif (has_post_thumbnail($product_id)) {
            $image = get_the_post_thumbnail($product_id, 'shop_thumbnail');
        } elseif (($parent_id = wp_get_post_parent_id($product_id)) && has_post_thumbnail($parent_id)) {
            $image = get_the_post_thumbnail($parent_id, 'shop_thumbnail');
        } else {
            $image = wc_placeholder_img('shop_thumbnail');
        }

        return $image;
    }

    private function get_variation_name($product, $order_item) {
        if (!$product) {
            return null;
        }

        $name = $product->get_title();
        if ($product->get_type() == 'variation') {
            foreach ($order_item->get_formatted_meta_data() as $meta_id => $meta) {
                $name .= ' - ' . $meta->display_key . ': ' . $meta->display_value;
            }
        }

        return $name;
    }

    protected function get_title($name, $product, $order_item) {
        if ($this->options->get_option('optimize-product')) {
            return $this->get_variation_name($product, $order_item) . ' ' . $this->get_line_items($order_item);
        }

        return $name . ' ' . $this->get_line_items($order_item) . ' ' . $this->get_attributes($product);
    }

    private function get_attributes($product) {
        if (!$product || !$this->options->get_option('show-attributes')) {
            return '';
        }

        $html = '';

        $attributes = $product->get_attributes();
        if (is_array($attributes) && count($attributes)) {
            $html .= '<ul class="product-attributes">';
            $i = 0;
            foreach ($attributes as $attribute) {
                if (!is_object($attribute) || !$attribute->get_visible()) {
                    continue;
                }
                $html .= '<li><span class="attribute-label">' . wc_attribute_label($attribute->get_name()) . ': </span>';
                $values = array();
                if ($attribute->is_taxonomy()) {
                    $attribute_taxonomy = $attribute->get_taxonomy_object();
                    $attribute_values = wc_get_product_terms($product->get_id(), $attribute->get_name(), array('fields' => 'all'));
                    foreach ($attribute_values as $attribute_value) {
                        $value_name = esc_html($attribute_value->name);
                        if ($attribute_taxonomy->attribute_public) {
                            $values[] = '<a href="' . esc_url(get_term_link($attribute_value->term_id, $attribute->get_name())) . '" rel="tag">' . $value_name . '</a>';
                        } else {
                            $values[] = $value_name;
                        }
                    }
                } else {
                    $values = $attribute->get_options();
                    foreach ($values as &$value) {
                        $value = make_clickable(esc_html($value));
                    }
                }
                $html .= '<span class="value">' . apply_filters('woocommerce_attribute', wptexturize(implode(', ', $values)), $attribute, $values) . '</span>';
                $html .= '</li>';
                if (count($attributes) > $i + 1) {
                    $html .= ' <span class="pipe">|</span> ';
                }
                $i++;
            }
            $html .= '</ul>';
        }

        return $html;
    }

    private function get_line_items($order_item) {
        $hidden_order_itemmeta = apply_filters(
            'woocommerce_hidden_order_itemmeta',
            array(
                '_qty',
                '_tax_class',
                '_product_id',
                '_variation_id',
                '_line_subtotal',
                '_line_subtotal_tax',
                '_line_total',
                '_line_tax',
                'method_id',
                'cost',
                '_reduced_stock',
            )
        );

        $line_items = '';

        if (!is_object($order_item)) {
            return $line_items;
        }

        if ($this->options->get_option('show-line-items')) {
            $meta_data = $order_item->get_formatted_meta_data('');
            if (count($meta_data)) {
                foreach ($meta_data as $data) {
                    if (in_array($data->key, $hidden_order_itemmeta, true)) {
                        continue;
                    }

                    if (isset($data->key)) {
                        $key = $data->key;
                    }
                    if (isset($data->display_key)) {
                        $key = $data->display_key;
                    }
                    if (isset($data->value)) {
                        $value = $data->value;
                    }
                    if (isset($data->display_value)) {
                        $value = $data->display_value;
                    }
                    $line_items .= '<span class="line-item">' . strip_tags($key) . ': ' . strip_tags($value) . '</span>' . ' ';
                }
            }
        }

        return $line_items;
    }

    protected function get_weight($product, $quantity) {
        if (!$product) {
            return null;
        }
        $product_weight = $product->get_weight();

        if (!$product_weight) {
            return '__';
        }

        return $product_weight . '&times;' . $quantity . ' = ' . wc_format_weight(($product_weight * $quantity));
    }

    protected function get_dimensions($product) {
        if (!$product) {
            return null;
        }
        $dimensions = wc_format_dimensions($product->get_dimensions(false));
        if (!$dimensions) {
            return '__';
        }

        return $dimensions;
    }

    protected function get_cabinet_code($product_id) {
        if (!intval($product_id)) {
            return null;
        }
        $product = wc_get_product($product_id);

        if ($product->get_type() == 'simple') {
            return get_post_meta($product_id, '_wip_cabinet_code', true);
        } elseif ($product->get_type() == 'variation') {
            return get_post_meta(wp_get_post_parent_id($product_id), '_wip_cabinet_code', true);
        }
    }
}